#ifndef KLIBLOADER_H
#define KLIBLOADER_H

#include <qobject.h>
#include <qstring.h>

#include "kstandarddirs.h"
#include "kglobal.h"

#include <iostream>
using std::cout;
using std::endl;

#ifdef WIN32
# include <windows.h>
#else
# include <dlfcn.h>
#endif

#define K_EXPORT_COMPONENT_FACTORY(X, Y) \
extern "C" { void *init_##X() { return new Y; } }

class KLibFactory: public QObject {
  Q_OBJECT
 public:
  KLibFactory() {}
  KLibFactory(QObject *parent, const char* name) {
  }
  virtual ~KLibFactory() {}

  virtual QObject* create(QObject* parent, const QString& text1,
		  const QString& text2) {
    return createObject(parent, text1, text2, "");
  }

  virtual QObject* createObject(QObject* parent, const char* name,
				const char* classname, const QStringList& args)  = 0;

  QString say() { return "Hello\n"; }
};

class KLibrary {
 public:
  KLibrary(KLibFactory *f): f(f) {}
  KLibrary() {}
  KLibFactory *factory() { return f; }
 private:
  KLibFactory *f;
};

class KLibLoader {
 public:
  void unloadLibrary(const QString& name) {
  }

  static KLibLoader* self() { 
    if(instance == NULL) {
      instance = new KLibLoader();
    }
    return instance; 
  }

  KLibrary* globalLibrary(const QString& libname) {
    tempLib = KLibrary(factory(libname));
    return &tempLib;
  }

  KLibFactory* factory(const QString& libname) {
    cout << "trying to load lib " << libname.latin1() << endl;
#ifdef WIN32
    QString name = libname + ".dll";
#else
    QString name = libname + ".so";
#endif
    QStringList list = 
      KGlobal::dirs()->findAllResources("module", name, false, true);
    if(list.count() == 0) { 
      cout << "Couldn't find library " << name.latin1() << endl;
      return 0; 
    }
#ifdef WIN32
#ifdef UNICODE
    HINSTANCE lib = LoadLibrary((LPCTSTR)(list.first() + QChar(0)).unicode());
#else
    HINSTANCE lib = LoadLibrary(list.first().latin1());
#endif
#else
    cout << list.first().latin1() << endl;
    void * lib = dlopen(list.first().latin1(),RTLD_LAZY | RTLD_GLOBAL);
#endif
    if(!lib) { 
#ifdef WIN32
      cout << "buuuuh" << endl; 
#else
      cout << dlerror() << endl;
#endif
      return 0; 
    }
    
    QString function = QString("init_") + libname;
#ifdef WIN32
    HMODULE hdll = (HMODULE) lib;
    void *symAddr = (void *) GetProcAddress(hdll, function.latin1());
#else
    void *symAddr = dlsym(lib, function.latin1());
#endif
    if(symAddr == NULL) { cout << "buuuuh2" << endl; return 0; }
    
    KLibFactory* fac = ((KLibFactory* (*)())symAddr)();
    
    cout << "got a fac..." << endl;
    return fac;
  }

  QString lastErrorMessage() { return ""; }
 private:
  static KLibLoader* instance;
  KLibrary tempLib;
};


#endif
