# xperthud.pl

# Example-Script that shows how to use the battletech-plugin

use XMBattleCore;
use XMBattleMapView;
use XMBattleContacts;
use XMBattleHeat;
use XMBattleWeapons;
use XMBattleSpeed;

use strict;
use warnings;
use diagnostics;

# GLOBAL vars for stuff that should be accessible from cmd-line-evals
use vars qw($reportChannel $battleCore $Core $Map $Contacts $Speed $Weapons);

statusWindow()->setWordWrapColumn(79);

$Core=$battleCore=new XMBattleCore("Core Debug");
$reportChannel="a";

setWriteRegExps(qr/^$/, qr/^#HUD/);

addTrigger("HUDINFO",qr/^#HUD/,sub {
	     my $line=shift;
	     $battleCore->parse($line);
	     return undef }
	   ,1);

$battleCore->startSecondary();

if (0) { #hrm... 
addTrigger("BT_TEXTPARSER", qr/^.*$/, sub {
              my $line=shift;
              $battleCore->parseText($line);
              return $line;
           },0, 0); # don't break, inactive.

}

addTrigger("TEXTPARSER_SKIP", qr/^.?/, sub {
	      my $line=shift;
	      $battleCore->parseText($line);
	      return undef
	   },1, 0); # break, inactive.

addTrigger("TEXTPARSER_START", qr/^SSKIP$/, sub {
	     setTriggerState(qr/^TEXTPARSER_SKIP$/,1);
	     return undef;
	   });

addTrigger("TEXTPARSER_END", qr/^ESKIP$/, sub {
	     setTriggerState(qr/^TEXTPARSER_SKIP$/,0);
	     return undef;
	   });

$Map=new XMBattleMapView("Map");
$Map->setZoom(10);
$Contacts=new XMBattleContacts("Contacts");
$Speed=new XMBattleSpeed("Speed");
$Weapons=new XMBattleWeapons("Weapons");

##################### Mode/Window layout control ###########################
sub normalMode {
  setKeyBindingState(qr/^BT_/, 0);
  setAliasState(qr/^BT_/, 0);
  setTriggerState(qr/^BT_/, 0);
  setTimerState(qr/^BT_/, 0);
  statusWindow()->maximize();

  $battleCore->setTacticalInterval(0);
  $battleCore->setStatusInterval(0);
  $battleCore->setContactsInterval(0);
  $battleCore->setWeaponsInterval(0);
}

sub battleMode {
  my $conHeight=160;
  my $speedWidth=50;
  my $weaponsHeight=160;

  setKeyBindingState(qr/^BT_/, 1);
  setAliasState(qr/^BT_/, 1);
  setTriggerState(qr/^BT_/, 1);
  setTimerState(qr/^BT_/, 1);

  statusWindow->hideCaption;
  statusWindow->resizeChars(81,40);
  statusWindow->move(0,$conHeight);
  statusWindow->resize(statusWindow->getSizeX, XM::mdiHeight()-$conHeight);

  $Contacts->hideCaption;
  $Contacts->move(0,0);
  $Contacts->resize(statusWindow->getSizeX-$speedWidth, $conHeight);
  $Contacts->show;

  $Speed->hideCaption;
  $Speed->move(statusWindow->getSizeX-$speedWidth,0);
  $Speed->resize($speedWidth,$conHeight);
  $Speed->show;

  $Map->hideCaption;
  $Map->move(statusWindow->getSizeX,0);
  $Map->resize(XM::mdiWidth() - statusWindow->getSizeX,
	       XM::mdiHeight()-$weaponsHeight);
  $Map->show;

  $Weapons->hideCaption;
  $Weapons->move(statusWindow->getSizeX,$Map->getSizeY);
  $Weapons->resize($Map->getSizeX,$weaponsHeight);
  $Weapons->show;


  $battleCore->setTacticalInterval(4*60);
  $battleCore->setStatusInterval(2);
  $battleCore->setContactsInterval(3);
  $battleCore->setWeaponsInterval(10);
}

################################# Claim Timer ##############################
{
  my $claimStart=0;
  my $claimTime=0;

  addTrigger("CLAIM_TIMER",
	     qr/^Complex claim initiated\. Claim will complete in \d+ minutes\./,
	     sub {
	       my $line = shift;
	       if($line =~ m/in (\d+)/) {
		 $claimTime=$1*60;
		 $claimStart=time;
		 statusWindow->setFGColor(BLUE);
		 statusWindow->print("Claim Noted.\n");
		 statusWindow->resetAttributes();
	       } $line;}, 0, 1);

  addAlias("CLAIM_TIMER",qr/^cclaim/,
	   sub {
	     if(time <  $claimStart + $claimTime) {
	       my $rest = $claimStart + $claimTime - time;
	       statusWindow->setFGColor(BLUE);
	       statusWindow->print(sprintf("Claim will be done in %02d:%02d\n",
					   (int($rest/60)),($rest%60)));
	       statusWindow->resetAttributes();
	     } else {
	       statusWindow->setFGColor(BLUE);
	       statusWindow->print("I dont know any ongoing Claims.\n");
	       statusWindow->resetAttributes();
	     }
	     return undef;
	   }, 1);

  addAlias("CLAIM_TIMER",qr/^setclaim \d+/,
	   sub {
	     my $line = shift;
	     if($line =~ m/setclaim (\d+)/) {
		 $claimTime=$1*60;
		 $claimStart=time;
		 statusWindow->setFGColor(BLUE);
		 statusWindow->print("Claim set.\n");
		 statusWindow->resetAttributes();
	       }
	     return undef;
	   } , 1);

  my $rclaimsub=sub {
    if(time <  $claimStart + $claimTime) {
      my $rest = $claimStart + $claimTime - time;
      XM::sendText(sprintf("sendchannel $reportChannel=TOC on Complex %02d:%02d\n",
		       (int($rest/60)),($rest%60)));
    } else {
      statusWindow->setFGColor(BLUE);
      statusWindow()->print("I dont know any ongoing Claims.\n");
      statusWindow->resetAttributes();
    }
    return undef;
  };

  addAlias("CLAIM_TIMER",qr/^rclaim/,$rclaimsub,1);

  addAlias("CLAIM_TIMER", qr/^autotoc(\s+\d+)?/,
	   sub {
	     my $interval=5*60;
	     my $line = shift;
	     if($line =~ /^autotoc\s+(\d+)/) {
	       $interval = $1*60;
	     }
	     if ($interval>0) {
	       if(time <  $claimStart + $claimTime) {
		 statusWindow->setFGColor(BLUE);
		 statusWindow->print("AutoTOC started. Will report every $interval seconds.\n");
		 statusWindow->resetAttributes();

		 delDelayed("^CLAIM_TIMER");
		 my $calltime = $claimStart + $claimTime-$interval;
		 while($calltime > time) {
		   addDelayed("CLAIM_TIMER",$calltime-time, $rclaimsub);
		   $calltime -= $interval;
		 }
	       } else {
		 statusWindow->setFGColor(BLUE);
		 statusWindow->print("I dont know any ongoing Claims.\n");
		 statusWindow->resetAttributes();
	       }
	     } else {
	       delDelayed("^CLAIM_TIMER");
	       statusWindow->setFGColor(BLUE);
	       statusWindow->print("AutoTOC disengaged.\n");
	       statusWindow->resetAttributes();
	     }
	     return undef;
	   } ,1);

}

############################## Mode Switches ##############################
# SHIFT CONTROL ALT META ALTGR NUML CAPSL SCRL

addKeyBinding("Switch_to_Battle","0000.... F6",sub { battleMode(); return 1; });
addKeyBinding("Switch_to_Normal","0000.... F5",sub { normalMode(); return 1; });
		

################################# Frequencies ##############################

addTrigger("START_FREQ","================================= Frequencies ================================",sub {
	     setTriggerState("^FREQ_",1);
	     return $_;
	   }, 0, 1);

addTrigger("END_FREQ",  "==============================================================================",sub {
	     setTriggerState("^FREQ_",0);
	     return $_;
	   }, 0, 1);

addTrigger("FREQ_MAIN","Main:.+(\\d+)",sub {
	     my $line=shift;
	     if ($line=~/(\d\d\d\d+)/) {
	       XM::sendText("&F_MAIN me=$1\n");
	     }
	     return $line;
	   }, 0,0);

addTrigger("FREQ_BACKUP1","Backup1:.+(\\d+)",sub {
	     my $line=shift;
	     if ($line=~/(\d\d\d\d+)/) {
	       XM::sendText("&F_BACKUP1 me=$1\n");
	     }
	     return $line;
	   }, 0,0);

addTrigger("FREQ_BACKUP2","Backup2:.+(\\d+)",sub {
	     my $line=shift;
	     if ($line=~/(\d\d\d\d+)/) {
	       XM::sendText("&F_BACKUP2 me=$1\n");
	     }
	     return $line;
	   }, 0,0);

addTrigger("FREQ_SCREAM1","Scream1:.+(\\d+)",sub {
	     my $line=shift;
	     if ($line=~/(\d\d\d\d+)/) {
	       XM::sendText("&F_SCREAM1 me=$1\n");
	     }
	     return $line;
	   }, 0,0);

addTrigger("FREQ_SCREAM2","Scream2:.+(\\d+)",sub {
	     my $line=shift;
	     if ($line=~/(\d\d\d\d+)/) {
	       XM::sendText("&F_SCREAM2 me=$1\n");
	     }
	     return $line;
	   }, 0,0);

addTrigger("FREQ_STRIKE","Strike/Recon:.+(\\d+)",sub {
	     my $line=shift;
	     if ($line=~/(\d\d\d\d+)/) {
	       XM::sendText("&F_STRIKE me=$1\n");
	     }
	     return $line;
	   }, 0,0);

addTrigger("FREQ_HEAVY","Heavy/Tank:.+(\\d+)",sub {
	     my $line=shift;
	     if ($line=~/(\d\d\d\d+)/) {
	       XM::sendText("&F_HEAVY me=$1\n");
	     }
	     return $line;
	   }, 0,0);

addTrigger("FREQ_CHAT","CHAT:.+(\\d+)",sub {
	     my $line=shift;
	     if ($line=~/(\d\d\d\d+)/) {
	       XM::sendText("&F_CHAT me=$1\n");
	     }
	     return $line;
	   }, 0,0);

{
  # Configure your prefered channel settings here
  my %mapping = (# primary channel, Modes, Backup channel, mode, backup slot
		 a=>["F_MAIN","DEG","F_BACKUP1","DER","f"],
		 b=>["F_SCREAM1","R","F_SCREAM2","R","g"],
		 c=>["F_CHAT","DEg"],
		 d=>["F_HEAVY","DEb"],
		 e=>["F_STRIKE","DEb"]
		);
  my $cmd="";
  foreach my $c (keys %mapping) {
    $cmd.="\@force me=setchannelfreq $c=[v($mapping{$c}->[0])]\n";
  }
  # TODO: keep them here, or set a character-script? or both?
  addAlias("SET_RADIO_FREQS","^setallfreqs",$cmd ,1);

  $cmd="";
  foreach my $c (keys %mapping) {
    my $mode=$mapping{$c}->[1];
    if ($mode=~/E/) {
      $mode =~ s/E//g;
      $cmd.="setchannelmode $c=$mode\n";
    }
    $cmd.="setchannelmode $c=$mapping{$c}->[1]\n";
  }
  addAlias("SET_RADIO_MODES","^setallmodes",$cmd ,1);

  $cmd="";
  foreach my $c (keys %mapping) {
    $cmd.="setchannelfreq $c=0\n";# or set random?
    if (defined $mapping{$c}->[4]) {
      $cmd.="setchannelfreq $mapping{$c}->[4]=0\n";
    }
  }
  addAlias("ZERO_RADIO_FREQS","^zeroallfreqs",$cmd,1);

  $cmd="";
  foreach my $c (keys %mapping) {
    next unless defined $mapping{$c}->[2];
    $cmd.="\@force me=setchannelfreq $c=[get(me/$mapping{$c}->[2])]\n";
    $cmd.="\@force me=setchannelfreq $mapping{$c}->[4]=[v($mapping{$c}->[0])]\n";
    my $mode=$mapping{$c}->[3];
    if ($mode=~/E/) {
      $mode =~ s/E//g;
      $cmd.="setchannelmode $c=$mode\n";
    }
    $cmd.="setchannelmode $mapping{$c}->[4]=$mapping{$c}->[3]\n";
  }
  addAlias("GO_RADIO_BACKUP","^setbackupfreqs",$cmd,1);

  $cmd ="\@force me=setchannelfreq a=[v($mapping{b}->[0])]\n";
  $cmd.="\@force me=setchannelmode a=$mapping{b}->[1]\n";
  addAlias("INIT_MECHWARRIOR","^initmw",$cmd,1);
  
}


#################################### IDLE ##################################

addDelayed("IDLE",25*60,sub {
	     XM::sendText("\@pemit me=The time is [time()].\n");
	     setDelayed(qr/^IDLE/,25*60);
	     normalMode();
	   });

########################### Movement keybindings ###########################
# SHIFT CONTROL ALT META ALTGR NUML CAPSL SCRL

addKeyBinding('BT_Heading-240', '.000.... KP_1',
	      sub { XM::sendText("heading 240\n"); return 1; });
addKeyBinding('BT_Heading-225', '.000.... KP_End',
	      sub { XM::sendText("heading 225\n"); return 1; });

addKeyBinding('BT_Heading-180', '.000.... KP_2',
	      sub { XM::sendText("heading 180\n"); return 1; });
addKeyBinding('BT_Heading-180', '.000.... KP_Down',
	      sub { XM::sendText("heading 180\n"); return 1; });

addKeyBinding('BT_Heading-120', '.000.... KP_3',
	      sub { XM::sendText("heading 120\n"); return 1; });
addKeyBinding('BT_Heading-135', '.000.... KP_PgDown',
	      sub { XM::sendText("heading 135\n"); return 1; });

addKeyBinding('BT_Heading-270', '.000.... KP_4',
	      sub { XM::sendText("heading 270\n"); return 1; });
addKeyBinding('BT_Heading-270', '.000.... KP_Left',
	      sub { XM::sendText("heading 270\n"); return 1; });

addKeyBinding('BT_Keypad_Stop', '.000.... KP_5',
	      sub { XM::sendText("speed stop\n"); return 1; });
addKeyBinding('BT_Keypad_Stop', '.000.... KP_Begin',
	      sub { XM::sendText("speed stop\n"); return 1; });

addKeyBinding('BT_Heading-90', '.000.... KP_6',
	      sub { XM::sendText("heading 90\n"); return 1; });
addKeyBinding('BT_Heading-90', '.000.... KP_Right',
	      sub { XM::sendText("heading 90\n"); return 1; });

addKeyBinding('BT_Heading-300', '.000.... KP_7',
	      sub { XM::sendText("heading 300\n"); return 1; });
addKeyBinding('BT_Heading-325', '.000.... KP_Home',
	      sub { XM::sendText("heading 325\n"); return 1; });

addKeyBinding('BT_Heading-0', '.000.... KP_8',
	      sub { XM::sendText("heading 0\n"); return 1; });
addKeyBinding('BT_Heading-0', '.000.... KP_Up',
	      sub { XM::sendText("heading 0\n"); return 1; });

addKeyBinding('BT_Heading-60', '.000.... KP_9',
	      sub { XM::sendText("heading 60\n"); return 1; });
addKeyBinding('BT_Heading-45', '.000.... KP_PgUp',
	      sub { XM::sendText("heading 45\n"); return 1; });

addKeyBinding('BT_Keypad_Run', '.000.... KP_0',
	      sub { XM::sendText("speed flank\n"); return 1; });
addKeyBinding('BT_Keypad_Run', '.000.... KP_Ins',
	      sub { XM::sendText("speed run\n"); return 1; });

addKeyBinding('BT_Keypad_Back', '.000.... KP_Del',
	      sub { XM::sendText("speed back\n"); return 1; });
addKeyBinding('BT_Keypad_Back', '.000.... KP_[,\.]',
	      sub { XM::sendText("speed back\n"); return 1; });


addKeyBinding('BT_Rottorso l', '.0(01|10).... KP_7',
	      sub { XM::sendText("rottorso l\n"); return 1; });
addKeyBinding('BT_Rottorso l', '.0(01|10).... KP_Home',
	      sub { XM::sendText("rottorso l\n"); return 1; });

addKeyBinding('BT_Rottorso c', '.0(01|10).... KP_8',
		  sub { XM::sendText("rottorso c\n"); return 1; });
addKeyBinding('BT_Rottorso c', '.0(01|10).... KP_Up',
		  sub { XM::sendText("rottorso c\n"); return 1; });

addKeyBinding('BT_Rottorso r', '.0(01|10).... KP_9',
	      sub { XM::sendText("rottorso r\n"); return 1; });
addKeyBinding('BT_Rottorso r', '.0(01|10).... KP_PgUp',
	      sub { XM::sendText("rottorso r\n"); return 1; });

########################## Targeting/Firing ################################
# SHIFT CONTROL ALT META ALTGR NUML CAPSL SCRL

addKeyBinding('BT_Target LL', '.100.... KP_1',
	      sub { XM::sendText("target LL\n"); return 1; });
addKeyBinding('BT_Target -',  '.100.... KP_2',
	      sub { XM::sendText("target -\n"); return 1; });
addKeyBinding('BT_Target RL', '.100.... KP_3',
	      sub { XM::sendText("target RL\n"); return 1; });
addKeyBinding('BT_Target LA', '.100.... KP_4',
	      sub { XM::sendText("target LA\n"); return 1; });
addKeyBinding('BT_Target CT', '.100.... KP_5',
	      sub { XM::sendText("target CT\n"); return 1; });
addKeyBinding('BT_Target RA', '.100.... KP_6',
	      sub { XM::sendText("target RA\n"); return 1; });
addKeyBinding('BT_Target LT', '.100.... KP_7',
	      sub { XM::sendText("target LT\n"); return 1; });
addKeyBinding('BT_Target H',  '.100.... KP_8',
	      sub { XM::sendText("target H\n"); return 1; });
addKeyBinding('BT_Target RT', '.100.... KP_9',
	      sub { XM::sendText("target RT\n"); return 1; });

addKeyBinding('BT_fire',  '.100.... (\d)$', #'
	      sub { XM::sendText("fire $1\n"); return 1; });
addKeyBinding('BT_sight', '.0(01|10).... (\d)$', #'
	      sub { XM::sendText("sight $2\n"); return 1; });

addKeyBinding('BT_fire0',  '.100.... asciicircum',
	      sub { XM::sendText("fire 0\n"); return 1; });
addKeyBinding('BT_fire0',  '.100.... \^',
	      sub { XM::sendText("fire 0\n"); return 1; });
addKeyBinding('BT_sight0', '.0(01|10).... asciicircum',
	      sub { XM::sendText("sight 0\n"); return 1; });
addKeyBinding('BT_sight0', '.0(01|10).... \^',
	      sub { XM::sendText("sight 0\n"); return 1; });

addKeyBinding('BT_firetic', '0000.... F([1234])', sub {
		my $tic = $1-1;
		XM::sendText("firetic $tic\n");
		return 1;
	      });
addKeyBinding('BT_listtic', '1000.... F([1234])', sub {
		my $tic = $1-1;
		XM::sendText("listtic $tic\n");
		return 1;
	      });

########################### Status / Scanning ###############################
# SHIFT CONTROL ALT META ALTGR NUML CAPSL SCRL

addKeyBinding('BT_Status', '.000.... KP_/',
	      sub { XM::sendText("status\n"); return 1; });

addKeyBinding('BT_Status', '.000.... KP_\*',
	      sub { XM::sendText("contacts\n"); return 1; });

addKeyBinding('BT_Keypad_Tactial', '.000.... KP_-',
	      sub { $Map->setZoom(15); return 1; });

addKeyBinding('BT_Keypad_Navigate', '.000.... KP_\+',
	      sub { $Map->setZoom(42); return 1; });

########################### General Macro Key ###############################

addKeyBinding('BT_shortcut_LC', '^0100.... (.)$', sub { #'
		XM::sendText('.'.lc($1)."\n"); return 1; }
);

addKeyBinding('BT_shortcut_UC', '^1100.... (.)$', sub { #'
		XM::sendText('.'.uc($1)."\n"); return 1; }
);


########################### Enter-Modes ####################################
# SHIFT CONTROL ALT META ALTGR NUML CAPSL SCRL

addAlias("Enter_to_PR","\n","\%r",0,1);
addAlias("Enter_to_SP","\n"," ",0,0);

addKeyBinding('Switch_Enter_PR','1000.... Return', sub {
		setKeyBindingState("Switch_Enter_PR",0);
		setKeyBindingState("Switch_Enter_SP",1);
		setAliasState("Enter_to_PR",1);
		setAliasState("Enter_to_SP",0);
		statusWindow->setFGColor(BLUE);
		statusWindow->setIntensive(1);
		statusWindow->print("Enter will generate \%r codes now\n");
		statusWindow->resetAttributes();
		return 1;
},0);

addKeyBinding('Switch_Enter_SP','1000.... Return', sub {
		setKeyBindingState("Switch_Enter_PR",1);
		setKeyBindingState("Switch_Enter_SP",0);
		setAliasState("Enter_to_PR",0);
		setAliasState("Enter_to_SP",1);
		statusWindow->setFGColor(BLUE);
		statusWindow->setIntensive(1);
		statusWindow->print("Enter will be replaced with space now\n");
		statusWindow->resetAttributes();
		return 1;
},1);

############################# AutoScroll ###################################

my $oldOnTextEntered=$onTextEntered;
$onTextEntered=sub {
  statusWindow->scrollLines(5000);
  &$oldOnTextEntered(@_);
  setDelayed("^IDLE\$",25*60);
};


############################### ConCaller ##################################

addAlias("DELETE_CONS","^clearcons",sub {
	   foreach($battleCore->getContacts) {
	     $battleCore->deleteContact($_);
	   }
	   return undef;
	 });

addAlias("BT_CONCALLER","^con",sub {
	   my $line=shift;
	   if ($line=~/^con\s+([A-Za-z][A-Za-z])/) {
	     my $mechid=$1;
	     my %me=$battleCore->getContact($mechid);
	     unless(defined $me{ID}) {
		statusWindow->setFGColor(BLUE);
		statusWindow->print("No Contact info on $mechid\n");
		statusWindow->resetAttributes();
		return undef;
	     }
	     my $s = sprintf(' [%s]%s %s x:%4d y:%4d z:%4d r:%5.1f '.
			     'b:%4d s:%5.1d h:%4d S: %s',
			     $me{ID},$me{UT},$me{MN},$me{X},$me{Y},
			     $me{Z},$me{RN},$me{BR},$me{SP},$me{HD},
			     $me{FL});
	     if($me{JH} ne "-") {
	       $s .= sprintf(' Jump h:%4d', $me{JH});
	     }
	     if($me{AG} > 0) {
	       $s .= sprintf(' Last Contact: %02d:%02d', int($me{AG}/60),
			     int($me{AG}%60));
	     } else {
	       $s .= " LOS maintained";
	     }
	     if(uc($me{ID}) eq $me{ID}) {
	       if($me{MN} eq "something") {
		 XM::sendText("sendchannel $reportChannel=* UNKNOWN * :".$s."\n");
	       } else {
		 XM::sendText("sendchannel $reportChannel=* ENEMY * :".$s."\n");
	       }
	     } else {
	       XM::sendText("sendchannel $reportChannel=FRIEND :".$s."\n");
	     }
	     return undef;
	   } else {
	     statusWindow->setFGColor(CYAN);
	     statusWindow->print("usage: con <mechid>\n");
	     statusWindow->resetAttributes();
	     return undef;
	   }
	 });

############################ Target Relock #################################

{
  my $lastLockedTarget="-";
  addTrigger("BT_ReLock",'Target set to .*\[(..)\]\.', sub { 
	       $lastLockedTarget=$1;
	       return shift;
	     },0,0);


  addKeyBinding('BT_ReLock', '.000.... KP_Enter', sub {
		  XM::sendText("lock $lastLockedTarget\n");
		  return 1;
		});

}
############################# Final inits ##################################

normalMode();


1;
