#include "PerlInterpret.h"

#include <cstdio>
// using snprintf;

// perl headers
extern "C" {
#  include <EXTERN.h>
#  include <perl.h>
#  include <XSUB.h>
}

#include "../GuiScriptingBindings.h"

#define NO_MULTILINE_STRING 1
#ifndef NO_MULTILINE_STRING
#include "sysinit_pl.h"
#else
#include "sysinit_plf.h"
#endif

#ifdef WIN32
#undef bool
#define snprintf _snprintf
#endif

// Stupid perl version incompatibilities...
#ifdef dXSTARG
#define FIXWARN
#else
#define dXSTARG
#endif

// TODO: add configure test! 5.6.1 compatibility mode as default
#undef FIXWARN

// Get rid of those stupid warnings
#ifdef FIXWARN
#define FIXPERLWARN  if(false) { cv+=0; Perl___notused+=0; }
#define FIXPERLWARN2  if (false) { targ+=0; }
#else
#define FIXPERLWARN
#define FIXPERLWARN2
#endif

// set from PerlInterpret while interpreting perl code
static GuiScriptingBindings *currentCallBack;

#define LOAD_ID \
    int id = -1; \
    if (SvROK(ST(0)) && SvTYPE(SvRV(ST(0)))==SVt_PVHV) { \
      HV* hash=(HV*)SvRV(ST(0)); \
      SV** sid=hv_fetch(hash,"_ID",3,0); \
      if (sid && SvIOK(*sid)) \
	id=SvIV(*sid); \
      else \
	croak("Invalid Parameter: Hash did not contain a int '_ID' key"); \
    } else \
      croak("Invalid Parameter: Object needs to be blessed Hashref");




extern "C" {
  void boot_DynaLoader (pTHX_ CV* cv);


  /////////////////////////////////////////////////////////////////////
  // Implementation XMWindow

  XS(XM_Window_setTitle) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMWindow->setTitle($text)");

    
    LOAD_ID;
    
    STRLEN len;
    char *title = SvPV(ST(1),len);
    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_setTitle(id,title, len);
    XSRETURN_YES;
  }

  XS(XM_Window_show) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->show");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_show(id);
    XSRETURN_YES;
  }

  XS(XM_Window_hide) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->hide");


    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_hide(id);
    XSRETURN_YES;
  }

  XS(XM_Window_close) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->close");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_close(id);
    XSRETURN_YES;
  }


  XS(XM_Window_minimize) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->minimize");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_minimize(id);
    XSRETURN_YES;
  }

  XS(XM_Window_maximize) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->maximize");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_maximize(id);
    XSRETURN_YES;
  }
  
  XS(XM_Window_isValid) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->isValid");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    if (currentCallBack->XM_Window_isValid(id)) {
      XSRETURN_YES;
    } else {
      XSRETURN_NO;
    }
  }
  
  XS(XM_Window_move) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 3)
      croak("Usage: XMWindow->move(x,y)");

    LOAD_ID;

    int x=SvIV(ST(1));
    int y=SvIV(ST(2));

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_move(id,x,y);

    XSRETURN_YES;
  }
 
  XS(XM_Window_resize) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 3)
      croak("Usage: XMWindow->resize(x,y)");

    LOAD_ID;

    int x=SvIV(ST(1));
    int y=SvIV(ST(2));

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_resize(id,x,y);

    XSRETURN_YES;
  }
 
  XS(XM_Window_raise) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->raise");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_raise(id);
    XSRETURN_YES;
  }

  XS(XM_Window_lower) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->lower");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_lower(id);
    XSRETURN_YES;
  }


  XS(XM_Window_getPositionX) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->getPositionX");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    int pos=currentCallBack->XM_Window_getPositionX(id);
    XSRETURN_IV(pos);
  }


  XS(XM_Window_getPositionY) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->getPositionY");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    int pos=currentCallBack->XM_Window_getPositionY(id);
    XSRETURN_IV(pos);
  }

  
  XS(XM_Window_getSizeX) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->getSizeX");

    LOAD_ID;
    
    dXSTARG;
    FIXPERLWARN2;

    int pos=currentCallBack->XM_Window_getSizeX(id);
    XSRETURN_IV(pos);
  }


  XS(XM_Window_getSizeY) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->getSizeY");

    LOAD_ID;


    dXSTARG;
    FIXPERLWARN2;

    int pos=currentCallBack->XM_Window_getSizeY(id);
    XSRETURN_IV(pos);
  }


  XS(XM_Window_hideCaption) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->hideCaption");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_hideCaption(id);
    XSRETURN_YES;
  }

  XS(XM_Window_showCaption) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMWindow->showCaption");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_Window_showCaption(id);
    XSRETURN_YES;
  }


  /////////////////////////////////////////////////////////////////////
  // Implementation XMTextWindow

  XS(XM_TextWindow_initialize) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 0)
      croak("Usage: XMTextWindowinitialize()");

    dXSTARG;
    FIXPERLWARN2;

    int id = currentCallBack->XM_TextWindow_initialize();

    XSRETURN_IV(id);
  }

  XS(XM_TextWindow_newline) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->newline");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_newline(id);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_clear) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->clear");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_clear(id);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_clearLine) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->clearLine");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_clearLine(id);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_clearEOL) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->clearEOL");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_clearEOL(id);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_resetAttributes) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->resetAttributes");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_resetAttributes(id);
    XSRETURN_YES;
  }
   
  XS(XM_TextWindow_printRaw) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->printRaw($text)");
    
    LOAD_ID;
    
    STRLEN len;
    char *text = SvPV(ST(1),len);
    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_printRaw(id,text, len);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_print) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->print($text)");
    
    LOAD_ID;
    
    STRLEN len;
    char *text = SvPV(ST(1),len);
    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_print(id,text, len);
    XSRETURN_YES;
  }
 
  XS(XM_TextWindow_setBGColor) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->setBGColor(color)");

    LOAD_ID;

    int color=SvIV(ST(1));
    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setBGColor(id,color);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_setDefaultBGColor) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->setDefaultBGColor(color)");

    LOAD_ID;

    int color=SvIV(ST(1));
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setDefaultBGColor(id,color);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_setFGColor) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->setFGColor(color)");

    LOAD_ID;

    int color=SvIV(ST(1));
    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setFGColor(id,color);
    XSRETURN_YES;
  }
  
  XS(XM_TextWindow_setDefaultFGColor) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->setDefaultFGColor(color)");

    LOAD_ID;

    int color=SvIV(ST(1));
    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setDefaultFGColor(id,color);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_scrollLines) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->scrollLines(howmuch)");

    LOAD_ID;

    int count=SvIV(ST(1));
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_scrollLines(id,count);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_scrollColumns) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->scrollColumns(howmuch)");

    LOAD_ID;

    int count=SvIV(ST(1));
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_scrollColumns(id,count);
    XSRETURN_YES;
  }

  
  XS(XM_TextWindow_setBold) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->setBold(bool)");

    LOAD_ID;

    bool is_it=(SvIV(ST(1)) != 0);
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setBold(id,is_it);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_setBlinking) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->setBlinking(bool)");

    LOAD_ID;

    bool is_it=(SvIV(ST(1)) != 0);
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setBlinking(id,is_it);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_setUnderline) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->setUnderline(bool)");

    LOAD_ID;

    bool is_it=(SvIV(ST(1)) != 0);
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setUnderline(id,is_it);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_setReverse) {
    dXSARGS;
    FIXPERLWARN;
    
    if (items != 2)
      croak("Usage: XMTextWindow->setReverse(bool)");

    LOAD_ID;

    bool is_it=(SvIV(ST(1)) != 0);
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setReverse(id,is_it);
    XSRETURN_YES;
  }
  
  XS(XM_TextWindow_setIntensive) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextWindow->setIntensive(bool)");

    LOAD_ID;

    bool is_it=(SvIV(ST(1)) != 0);
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setIntensive(id,is_it);
    XSRETURN_YES;
  }


  XS(XM_TextWindow_setCursor) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 3)
      croak("Usage: XMTextWindow->setCursor(x,y)");

    LOAD_ID;

    unsigned int x=SvUV(ST(1));
    unsigned int y=SvUV(ST(2));
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_setCursor(id,x,y);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_getCursorX) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->getCursorX");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    int pos=currentCallBack->XM_TextWindow_getCursorX(id);
    XSRETURN_IV(pos);
  }

  XS(XM_TextWindow_getCursorY) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->getCursorY");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    int pos=currentCallBack->XM_TextWindow_getCursorY(id);
    XSRETURN_IV(pos);
  }
  


  XS(XM_TextWindow_getColumns) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->getColumns");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    int pos=currentCallBack->XM_TextWindow_getColumns(id);
    XSRETURN_IV(pos);
  }
  


  XS(XM_TextWindow_getLines) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->getLines");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    int pos=currentCallBack->XM_TextWindow_getLines(id);
    XSRETURN_IV(pos);
  }
  
  XS(XM_TextWindow_setFont) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 3)
      croak("Usage: XMTextWindow->setFont($text,size)");
    
    LOAD_ID;
    
    STRLEN len;
    char *text = SvPV(ST(1),len);

    int size=SvIV(ST(2));

    dXSTARG;
     FIXPERLWARN2;

     if (currentCallBack->XM_TextWindow_setFont(id,text, len,size)) {
       XSRETURN_YES;
     } else {
       XSRETURN_NO;
     }
  }


  XS(XM_TextWindow_getCharAt) {
    dXSARGS;
    FIXPERLWARN;
    
    if (items != 3)
      croak("Usage: XMTextWindow->getCharAt(x,y)");
    
    LOAD_ID;
    
    int x=SvIV(ST(1));
    int y=SvIV(ST(2));
    
    dXSTARG;
    FIXPERLWARN2;
    
    char str[2];
    str[0]=currentCallBack->XM_TextWindow_getCharAt(id,x,y);
    str[1]=0;
    // ifdef perl5.6    
    //    XSRETURN_PVN(str,1);
    // else
    XSRETURN_PV(str);
  }
  
  XS(XM_TextWindow_getFGColorAt) {
    dXSARGS;
    FIXPERLWARN;
    
    if (items != 3)
      croak("Usage: XMTextWindow->getFGColorAt(x,y)");
    
    LOAD_ID;
    
    int x=SvIV(ST(1));
    int y=SvIV(ST(2));
    
    dXSTARG;
    FIXPERLWARN2;
    
    int color = currentCallBack->XM_TextWindow_getFGColorAt(id,x,y);
    XSRETURN_IV(color);
  }  

  XS(XM_TextWindow_getBGColorAt) {
    dXSARGS;
    FIXPERLWARN;
    
    if (items != 3)
      croak("Usage: XMTextWindow->getBGColorAt(x,y)");
    
    LOAD_ID;
    
    int x=SvIV(ST(1));
    int y=SvIV(ST(2));
    
    dXSTARG;
    FIXPERLWARN2;
    
    int color = currentCallBack->XM_TextWindow_getBGColorAt(id,x,y);
    XSRETURN_IV(color);
  }  

  XS(XM_TextWindow_resizeChars) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 3)
      croak("Usage: XMTextWindow->resizeChars(x,y)");

    LOAD_ID;

    int x=SvIV(ST(1));
    int y=SvIV(ST(2));

    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextWindow_resizeChars(id,x,y);

    XSRETURN_YES;
  }

  XS(XM_TextWindow_enableMouseEvents) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->enableMouseEvents");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_enableMouseEvents(id);
    XSRETURN_YES;
  }

  XS(XM_TextWindow_disableMouseEvents) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextWindow->disableMouseEvents");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_TextWindow_disableMouseEvents(id);
    XSRETURN_YES;
  }

  /////////////////////////////////////////////////////////////////////
  // Implementation XMTextBufferWindow
  
  XS(XM_TextBufferWindow_initialize) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 0)
      croak("Usage: XMTextBufferWindow::initialize()");

    dXSTARG;
    FIXPERLWARN2;

    int id = currentCallBack->XM_TextBufferWindow_initialize();

    XSRETURN_IV(id);
  }

  XS(XM_TextBufferWindow_getWordWrapColumn) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XMTextBufferWindow->getWordWrapColumn");

    LOAD_ID;

    dXSTARG;
    FIXPERLWARN2;

    int pos=currentCallBack->XM_TextBufferWindow_getWordWrapColumn(id);
    XSRETURN_IV(pos);
  }
  
  XS(XM_TextBufferWindow_setWordWrapColumn) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMTextBufferWindow->setWordWrapColumn(int)");

    LOAD_ID;

    int col=SvIV(ST(1));

    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_TextBufferWindow_setWordWrapColumn(id,col);
    XSRETURN_YES;
  }

  /////////////////////////////////////////////////////////////////////
  // Implementation XMPlugin
  
  XS(XM_Plugin_initialize) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 2)
      croak("Usage: XMPlugin::initialize(lib,class)");

    STRLEN liblen;
    char *libname = SvPV(ST(0),liblen);

    STRLEN classlen;
    char *classname = SvPV(ST(1),classlen);

    dXSTARG;
    FIXPERLWARN2;


    int id = currentCallBack->XM_Plugin_initialize(libname,liblen,
						   classname,classlen);

    XSRETURN_IV(id);
  }
  
  
  XS(XM_Plugin_call) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 3)
      croak("Usage: XMPlugin::call(fnum,params)");

    LOAD_ID;

    int fnum=SvIV(ST(1));

    STRLEN paramlen;
    char *param = SvPV(ST(2),paramlen);

    dXSTARG;
    FIXPERLWARN2;

    char * result=currentCallBack->XM_Plugin_call(id,fnum,param,paramlen);
    
    if (result) {
      ST(0)=sv_2mortal(newSVpv(result,0));
      delete [] result;
      XSRETURN(1);
    } else {
      XSRETURN_UNDEF;
    }
  }


  /////////////////////////////////////////////////////////////////////
  // Implementation XM

  XS(XM_send) {
    dXSARGS;
    FIXPERLWARN;

    if (items < 1 || items > 2)
      croak("Usage: XM::send($text, $id=0)");

    STRLEN len;
    char *input1 = SvPV(ST(0),len);
    int input2 = 0;
    if(items == 2)
      input2 = SvIV(ST(1));
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_send(input1, len, input2);
    XSRETURN_UNDEF;
  }

  XS(XM_close) {
    dXSARGS;
    FIXPERLWARN;

    if (items > 1)
      croak("Usage: XM::close($id=0)");

    int input1 = 0;
    if(items == 1)
      input1 = SvIV(ST(0));
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_close(input1);
    XSRETURN_UNDEF;
  }

  XS(XM_open) {
    dXSARGS;
    FIXPERLWARN;

    if (items < 2 || items > 4)
      croak("Usage: XM::open($host, $port, $id=0, $onConnect=\"\")");

    STRLEN len;
    char *input1 = SvPV(ST(0),len);
    int input2 = SvIV(ST(1));
    int input3 = 0;
    if(items >= 3)
      input3 = SvIV(ST(2));
    STRLEN len4 = 0;
    char *input4 = "";
    if(items >= 4)
      input4 = SvPV(ST(3),len4);
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_open(input1, len, input2, input3,
			       input4, len4);
    XSRETURN_UNDEF;
  }
  
  XS(XM_setStatusMsg) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XM::setStatusMsg($msg)");

    STRLEN len;
    char *input1 = SvPV(ST(0),len);
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_setStatusMsg(input1,len);
    XSRETURN_UNDEF;
    
  }

  XS(XM_resetStatusMsg) {
    currentCallBack->XM_resetStatusMsg();
    if (false) { cv+=0; }
  }

  XS(XM_showError) {
    dXSARGS;
    FIXPERLWARN;
    
    if (items != 1)
      croak("Usage: XM::setStatusMsg($msg)");

    STRLEN len;
    char *input1 = SvPV(ST(0),len);
    dXSTARG;
     FIXPERLWARN2;

    currentCallBack->XM_showError(input1,len);
    XSRETURN_UNDEF;
  }

  XS(XM_echoMode) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 1)
      croak("Usage: XM::echoMode($state)");

    bool state=(SvIV(ST(0)) != 0);
    dXSTARG;
    FIXPERLWARN2;

    currentCallBack->XM_echoMode(state);
    XSRETURN_UNDEF;
  }


  // Needs Perl >= 5.6.0 (or patched 5.0)

  XS(XM_weaken) {
    dXSARGS;
    FIXPERLWARN;
    if (items != 1)
      croak("Usage: XM::weaken(ref)");

    if (SvROK(ST(0)))
      sv_rvweaken(ST(0));
    else
      croak("Usage: XM::weaken(REF)");

    dXSTARG;
    FIXPERLWARN2;
    XSRETURN_YES;
  }

  XS(XM_mdiWidth) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 0)
      croak("Usage: XM::mdiWidth");


    dXSTARG;
    FIXPERLWARN2;

    int w=currentCallBack->XM_mdiWidth();
    XSRETURN_IV(w);
  }

  XS(XM_mdiHeight) {
    dXSARGS;
    FIXPERLWARN;

    if (items != 0)
      croak("Usage: XM::mdiHeight");


    dXSTARG;
    FIXPERLWARN2;

    int h=currentCallBack->XM_mdiHeight();
    XSRETURN_IV(h);
  }


  /////////////////////////////////////////////////////////////////////
  // Activation


  void xs_init(pTHX) {
    newXS((char *)"DynaLoader::boot_DynaLoader", boot_DynaLoader, (char *)__FILE__); 

    newXS((char *)"XMWindow::setTitle",
	  XM_Window_setTitle , (char *)__FILE__); 
    newXS((char *)"XMWindow::show",
	  XM_Window_show , (char *)__FILE__); 
    newXS((char *)"XMWindow::hide",
	  XM_Window_hide , (char *)__FILE__); 
    newXS((char *)"XMWindow::close",
	  XM_Window_close , (char *)__FILE__); 
    newXS((char *)"XMWindow::minimize",
	  XM_Window_minimize , (char *)__FILE__); 
    newXS((char *)"XMWindow::maximize",
	  XM_Window_maximize , (char *)__FILE__); 
    newXS((char *)"XMWindow::move",
	  XM_Window_move , (char *)__FILE__); 
    newXS((char *)"XMWindow::getPositionX",
	  XM_Window_getPositionX , (char *)__FILE__); 
    newXS((char *)"XMWindow::getPositionY",
	  XM_Window_getPositionY , (char *)__FILE__); 
    newXS((char *)"XMWindow::getSizeX",
	  XM_Window_getSizeX , (char *)__FILE__);
    newXS((char *)"XMWindow::getSizeY",
	  XM_Window_getSizeY , (char *)__FILE__);
    newXS((char *)"XMWindow::resize",
	  XM_Window_resize , (char *)__FILE__);
    newXS((char *)"XMWindow::isValid",
	  XM_Window_isValid , (char *)__FILE__); 
    newXS((char *)"XMWindow::raise",
	  XM_Window_raise , (char *)__FILE__); 
    newXS((char *)"XMWindow::lower",
	  XM_Window_lower , (char *)__FILE__); 
    newXS((char *)"XMWindow::hideCaption",
	  XM_Window_hideCaption , (char *)__FILE__); 
    newXS((char *)"XMWindow::showCaption",
	  XM_Window_showCaption , (char *)__FILE__); 

    newXS((char *)"XMTextWindow::initialize",
	  XM_TextWindow_initialize , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::newline",
	  XM_TextWindow_newline , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::clear",
	  XM_TextWindow_clear , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::clearLine",
	  XM_TextWindow_clearLine , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::clearEOL",
	  XM_TextWindow_clearEOL , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::resetAttributes",
	  XM_TextWindow_resetAttributes , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::printRaw",
	  XM_TextWindow_printRaw , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::print",
	  XM_TextWindow_print , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setBGColor",
	  XM_TextWindow_setBGColor , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setDefaultBGColor",
	  XM_TextWindow_setDefaultBGColor , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setFGColor",
	  XM_TextWindow_setFGColor , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setDefaultFGColor",
	  XM_TextWindow_setDefaultFGColor , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::scrollLines",
	  XM_TextWindow_scrollLines , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::scrollColumns",
	  XM_TextWindow_scrollColumns , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setBold",
	  XM_TextWindow_setBold , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setBlinking",
	  XM_TextWindow_setBlinking , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setUnderline",
	  XM_TextWindow_setUnderline , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setReverse",
	  XM_TextWindow_setReverse , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setIntensive",
	  XM_TextWindow_setIntensive , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setCursor",
	  XM_TextWindow_setCursor , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::getCursorX",
	  XM_TextWindow_getCursorX , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::getCursorY",
	  XM_TextWindow_getCursorY , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::resizeChars",
	  XM_TextWindow_resizeChars , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::getCharAt",
	  XM_TextWindow_getCharAt , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::setFont",
	  XM_TextWindow_setFont , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::getLines",
	  XM_TextWindow_getLines , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::getColumns",
	  XM_TextWindow_getColumns , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::getFGColorAt",
	  XM_TextWindow_getFGColorAt , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::getBGColorAt",
	  XM_TextWindow_getBGColorAt , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::enableMouseEvents",
	  XM_TextWindow_enableMouseEvents , (char *)__FILE__); 
    newXS((char *)"XMTextWindow::disableMouseEvents",
	  XM_TextWindow_disableMouseEvents , (char *)__FILE__); 

    newXS((char *)"XMTextBufferWindow::initialize",
	  XM_TextBufferWindow_initialize , (char *)__FILE__); 

    // ... not implemented in python
    newXS((char *)"XMTextBufferWindow::getWordWrapColumn",
	  XM_TextBufferWindow_getWordWrapColumn , (char *)__FILE__); 
    newXS((char *)"XMTextBufferWindow::setWordWrapColumn",
	  XM_TextBufferWindow_setWordWrapColumn , (char *)__FILE__); 
    // ...

    newXS((char *)"XMPlugin::initialize",
	  XM_Plugin_initialize , (char *)__FILE__); 
    newXS((char *)"XMPlugin::call",
	  XM_Plugin_call , (char *)__FILE__); 

    newXS((char *)"XM::send",
	  XM_send , (char *)__FILE__); 
    newXS((char *)"XM::open",
	  XM_open , (char *)__FILE__); 
    newXS((char *)"XM::close",
	  XM_close , (char *)__FILE__); 
    newXS((char *)"XM::setStatusMsg",
	  XM_setStatusMsg , (char *)__FILE__); 
    newXS((char *)"XM::resetStatusMsg",
	  XM_resetStatusMsg , (char *)__FILE__); 
    newXS((char *)"XM::showError",
	  XM_showError , (char *)__FILE__); 
    newXS((char *)"XM::echoMode",
	  XM_echoMode , (char *)__FILE__); 
    newXS((char *)"XM::mdiWidth",
	  XM_mdiWidth , (char *)__FILE__); 
    newXS((char *)"XM::mdiHeight",
	  XM_mdiHeight , (char *)__FILE__); 

    // ... this is a std function in python 
    newXS((char *)"XM::weaken",
	  XM_weaken , (char *)__FILE__); 
    // ...
    
    
    
  }
}
#if 0
} // Sorry about that... is some emacs indent hack 
#endif

/////////////////////////////////////////////////////////////////////
// Implementation class PerlInterpret


PerlInterpret::PerlInterpret(GuiScriptingBindings *callBack):
  callBack(callBack)
{
  currentCallBack = callBack;

  perlInt = perl_alloc();
  
  if (!perlInt)
    exit(42);

  PL_perl_destruct_level = 1;

  perl_construct(perlInt);
  const char *args[]={"xperlmud",
		      "-w",
		      "-e", ""
  };
  int ret = perl_parse(perlInt,xs_init,4,(char **)args,0);
  if(!ret)
    perl_run(perlInt);
  else 
    exit(23);

#ifndef SCRIPTDEVEL
# ifndef NO_MULTILINE_STRING
  printf("evaluating sysinit (multi-line string):\n");
  eval_pv(SYSINIT, true);
# else
  printf("evaluating sysinit (no multi-line string):\n");
  int i=0;
  int sum=0;
  while(script[i] != NULL) {
    sum += strlen(script[i]);
    ++i;
  }
  char *commandString = new char[sum+1];
  i=0;
  int c=0;
  while(script[i] != NULL) {
    int y=0;
    while(script[i][y] != 0) {
      commandString[c] = script[i][y];
      ++y; ++c;
    }
    ++i;
  }
  commandString[c] = 0;
  eval_pv(commandString, false);

  delete[] commandString;
# endif
#else
  printf("evaluating sysinit (FLEXIBLE):\n");
  eval_pv("do 'sysinit.pl';", true);
#endif
  //  STRLEN n_a;
  //  printf("finished.\"%s\" %d\n", SvPV(get_sv("@", FALSE), n_a), n_a);
}

PerlInterpret::~PerlInterpret() {
  currentCallBack = callBack;

  perl_destruct(perlInt);
  perl_free(perlInt);
}

void PerlInterpret::setCallback(GuiScriptingBindings *cB) {
  currentCallBack = callBack = cB;
}
  
  
// TODO
void PerlInterpret::textEntered(const char * const text) {
    currentCallBack = callBack;
  const char *args[]={text,0};

  ENTER;
  SAVETMPS;
  perl_call_argv("::wrap_onTextEntered",G_DISCARD,(char **)args);
  FREETMPS;
  LEAVE;
}

// TODO
void PerlInterpret::addIncludeDir(const char * const inc) {
  currentCallBack = callBack;
  const char *args[]={inc,0};

  ENTER;
  SAVETMPS;
  perl_call_argv("::addIncludeDir",G_DISCARD,(char **)args);
  FREETMPS;
  LEAVE;
}

void PerlInterpret::textReceived(const char * const text, int length, int id) {
  currentCallBack = callBack;

  dSP;

  ENTER;
  SAVETMPS;
  PUSHMARK(SP);
  XPUSHs(sv_2mortal(newSVpv(text, length)));
  XPUSHs(sv_2mortal(newSViv(id)));
  PUTBACK;

  call_pv("::wrap_onTextReceived",G_DISCARD);

  SPAGAIN;

  PUTBACK;
  FREETMPS;
  LEAVE;

}

void PerlInterpret::timer() {
  currentCallBack = callBack;

  dSP;

  ENTER;
  SAVETMPS;
  PUSHMARK(SP);

  PUTBACK;

  call_pv("::wrap_onTimer",G_DISCARD);

  SPAGAIN;

  PUTBACK;
  FREETMPS;
  LEAVE;
}

// TODO
void PerlInterpret::connectionEstablished(int id) {
  currentCallBack = callBack;
  char ids[25];
  snprintf(ids,sizeof(ids),"%d",id);
  char *args[]={ids,0};

  ENTER;
  SAVETMPS;
  perl_call_argv("::wrap_onConnectionEstablished",G_DISCARD,args);
  FREETMPS;
  LEAVE;
}

// TODO
void PerlInterpret::connectionDropped(int id) {
  currentCallBack = callBack;
  char ids[25];
  snprintf(ids,sizeof(ids),"%d",id);
  char *args[]={ids,0};

  ENTER;
  SAVETMPS;
  perl_call_argv("::wrap_onConnectionDropped",G_DISCARD,args);
  FREETMPS;
  LEAVE;
}

// TODO
void PerlInterpret::echo(bool state, int id) {
  currentCallBack = callBack;
  char ids[25];
  snprintf(ids,sizeof(ids),"%d",id);
  const char *args[]={state?"1":"0",const_cast<const char *>(ids),0};
  
  ENTER;
  SAVETMPS;
  perl_call_argv("wrap_onEcho",G_DISCARD,(char **)args);
  FREETMPS;
  LEAVE;
}

// TODO
bool PerlInterpret::keyPressed(const char * const key, 
			       const char * const ascii) {
  bool ret;
  dSP;
#ifdef FIXWARN
  if (false) { Perl___notused+=0; }
#endif
  currentCallBack = callBack;
  const char *args[]={key,ascii,0};

  ENTER;
  SAVETMPS;

  //  PUSHMARK(SP) ;
  //  PUTBACK;

  int count = perl_call_argv("::wrap_onKeyPressed",G_SCALAR,(char **)args);
  if(count != 1)
     croak("Big trouble\n") ;
  
  SPAGAIN;
  ret = (POPi != 0);

  PUTBACK;
  FREETMPS;
  LEAVE;

  return ret;
}

// TODO
void PerlInterpret::mouseDown(int id,int x,int y) {
  currentCallBack = callBack;

  char ids[25];
  char xs[25];
  char ys[25];
  snprintf(ids,sizeof(ids),"%d",id);
  snprintf(xs,sizeof(xs),"%d",x);
  snprintf(ys,sizeof(ys),"%d",y);

  const char *args[]={ids,xs,ys,0};
    
  ENTER;
  SAVETMPS;
  perl_call_argv("::wrap_mouseDown",G_DISCARD,(char **)args);
  FREETMPS;
  LEAVE;
}

// TODO
void PerlInterpret::mouseDrag(int id,int x,int y) {
  currentCallBack = callBack;

  char ids[25];
  char xs[25];
  char ys[25];
  snprintf(ids,sizeof(ids),"%d",id);
  snprintf(xs,sizeof(xs),"%d",x);
  snprintf(ys,sizeof(ys),"%d",y);

  const char *args[]={ids,xs,ys,0};
    
  ENTER;
  SAVETMPS;
  perl_call_argv("::wrap_mouseDrag",G_DISCARD,(char **)args);
  FREETMPS;
  LEAVE;
}

// TODO
void PerlInterpret::mouseUp(int id,int x,int y) {
  currentCallBack = callBack;

  char ids[25];
  char xs[25];
  char ys[25];
  snprintf(ids,sizeof(ids),"%d",id);
  snprintf(xs,sizeof(xs),"%d",x);
  snprintf(ys,sizeof(ys),"%d",y);

  const char *args[]={ids,xs,ys,0};
    
  ENTER;
  SAVETMPS;
  perl_call_argv("::wrap_mouseUp",G_DISCARD,(char **)args);
  FREETMPS;
  LEAVE;
}

const char * PerlInterpret::pluginCall(int id,int function, const char * const args) {
  currentCallBack = callBack;
  dSP;

  ENTER;
  SAVETMPS;
  PUSHMARK(SP);
  XPUSHs(sv_2mortal(newSViv(id)));
  XPUSHs(sv_2mortal(newSViv(function)));
  XPUSHs(sv_2mortal(newSVpv(args,0)));
  PUTBACK;

  int count = call_pv("::wrap_pluginCall",G_SCALAR);

  SPAGAIN;

  if (count != 1)
    croak("Big trouble\n") ;

  SV * rval=POPs;

  STRLEN len;
  char *retval_temp = SvPV(rval,len);

  char * retval=new char[len+1];
  strncpy(retval,retval_temp,len);
  retval[len]=0;

  PUTBACK;
  FREETMPS;
  LEAVE;
  return retval;
}
