#!/usr/bin/perl -w
use strict;

my $SRCDIR = $ARGV[1];

my @modules;
my @types;
my @topfunctions;
my $topLevelRef; # modules, classes
my $firstLevelRef; # functions, variables
my $insideRef;
my $insideRefMod = undef;

sub parseLine {
  shift;
  s/\(py\?([^\)]*)\)//g;
  s/\(pe\?([^\)]*)\)/$1/g;
  if(/^py>(.*)$/) {
    $_ = "";
  }
  if(/^pe>(.*)$/) {
    $_ = $1."\n";
  }
  if(/^___INCLUDE (.*)$/) {
    open(INCFILE, "<".$SRCDIR."/$1");
    while(<INCFILE>) {
      parseLine($_);
    }
    close INCFILE;
  } elsif(/<module>\s*([\w\d_]+)/) {
    $firstLevelRef = [];
    $topLevelRef = [ "$1", $firstLevelRef, "Module", "", "" ];
    $insideRef = $topLevelRef;
    $insideRefMod = 1;
    push(@modules, $topLevelRef);

  } elsif(/<class>\s*([\w\d_]+)/) {
    $firstLevelRef = [];
    $topLevelRef = [ "$1", $firstLevelRef, "Class", "", "" ];
    $insideRef = $topLevelRef;
    $insideRefMod = 1;
    push(@modules, $topLevelRef);

  } elsif(/<topic>\s*([^\s].+)$/) {
    $firstLevelRef = [];
    $topLevelRef = [ "$1", $firstLevelRef, "Topic", "", "" ];
    $insideRef = $topLevelRef;
    $insideRefMod = 1;
    push(@modules, $topLevelRef);

  } elsif(/<function>\s*([\w\d_]+)\(([^\)]*)\)/) {
    my $funName = $1;
    my $argStr = $2;
    $argStr =~ s/\"/\\\"/g;
    $insideRef = [ "$funName", "$argStr", "", "" ];
    $insideRefMod = 0;
    push(@$firstLevelRef, $insideRef);

  } elsif(/<subtopic>\s*([^\s].+)$/) {
    $insideRef = [ "$1", "", "", "" ];
    $insideRefMod = 0;
    push(@$firstLevelRef, $insideRef);

  } elsif(/<mainfunction>\s*([\w\d\#\%_]+)\(([^\)]*)\)/) {
    my $funName = $1;
    my $argStr = $2;
    $argStr =~ s/\"/\\\"/g;
    $insideRef = [ "$funName", "$argStr", "", "" ];
    $insideRefMod = 0;
    push(@topfunctions, $insideRef);

  } else {
    if(!$insideRefMod) {
      my $db = $_;
      $db =~ s/&/and/g;
      $db =~ s/</&lt;/g;
      $db =~ s/>/&gt;/g;
      $$insideRef[3] .= $db;
      s/\\/\\\\/g;
      s/#/\\#/g;
      s/\"/\\\"/g;
      s/\$/\\\$/g;
      s/\@/\\\@/g;
      $$insideRef[2] .= $_;
    } else {
      my $db = $_;
      $db =~ s/&/and/g;
      $db =~ s/</&lt;/g;
      $db =~ s/>/&gt;/g;
      $$insideRef[4] .= $db;
      s/\\/\\\\/g;
      s/#/\\#/g;
      s/\"/\\\"/g;
      s/\$/\\\$/g;
      s/\@/\\\@/g;
      $$insideRef[3] .= $_;
    }
  }
}

while(<STDIN>) {
  parseLine($_);
}

############################################################
############################################################
############################################################

sub subOnlineColors {
  my $text = shift;
  $text =~ s/\%cn/\x1b[0m/g;
  $text =~ s/\%cx/\x1b[30m/g;
  $text =~ s/\%cr/\x1b[31m/g;
  $text =~ s/\%cg/\x1b[32m/g;
  $text =~ s/\%cy/\x1b[33m/g;
  $text =~ s/\%cb/\x1b[34m/g;
  $text =~ s/\%cm/\x1b[35m/g;
  $text =~ s/\%cc/\x1b[36m/g;
  $text =~ s/\%cw/\x1b[37m/g;
  $text =~ s/\%cX/\x1b[40m/g;
  $text =~ s/\%cR/\x1b[41m/g;
  $text =~ s/\%cG/\x1b[42m/g;
  $text =~ s/\%cY/\x1b[43m/g;
  $text =~ s/\%cB/\x1b[44m/g;
  $text =~ s/\%cM/\x1b[45m/g;
  $text =~ s/\%cC/\x1b[46m/g;
  $text =~ s/\%cW/\x1b[47m/g;

  $text =~ s/\%ch/\x1b[1m/g;

  return $text;
}

sub createOnlineEntries {
  my @ret = ( 0, "" );
  my ($prefix, $type, $heading, $typeElement, $nameElement, $elements) = @_;

  my $numInLine = 0;
  $ret[1] = "$heading:\n";
  my $lastLen = 0;
  foreach my $element (@$elements) {
    #print "  type: $$element[$typeElement]\n";
    if($type eq "" || $$element[$typeElement] eq $type) {
      if($numInLine > 2) {
	$numInLine = 0;
	$ret[1] .= "\n\t";
      } elsif($numInLine > 0) {
	if($lastLen >= 16) {
	  $ret[1] .= "\t";
	} elsif($lastLen >= 8) {
	  $ret[1] .= "\t\t";
	} else {
	  $ret[1] .= "\t\t\t";
	}
      } else {
	$ret[1] .= "\t";
      }
      my $id = $$element[$nameElement];
      $id =~ s/ /_/g;
      $ret[1] .= "$$element[$nameElement]";
      $numInLine++;
      $lastLen = length($$element[$nameElement]);
      $ret[0]++;
    }
  }
  $ret[1] .= "\n\n";
  if($ret[0] == 0) {
    $ret[1] = "";
  }

  return @ret;
}

if($ARGV[0] eq "online") {
  print "# **********************************************\n";
  print "# *** DONT EDIT, GENERATED BY DESC2ONLINE.PL ***\n\n";

  my %num;
  my %numString;
  for my $s (['Module', 'Modules'],
	     ['Class', 'Classes'],
	     ['Topic', 'Topics']) {
    my ($type, $heading) = @$s;
    ($num{$type}, $numString{$type}) =
      createOnlineEntries("Perl_", $type, $heading, 2, 0, \@modules);
  }

  (my $funInS, my $functionsS) =
    createOnlineEntries("Perl_", "", "Functions", undef, 0, \@topfunctions);
  print <<EOS;
  if(!(defined \$helpOn)) {
    statusWindow()->print("
Xpertmud Online Help System (Perl)
**********************************

for help on a scpecific topic type
#help \\"TOPIC\\"

$functionsS$numString{'Class'}$numString{'Module'}$numString{'Topic'}");
EOS

  foreach my $module (@modules) {
    my $id = "Perl_" . $$module[0];
    $id =~ s/ /_/g;

    my $subname = "Functions";
    if($$module[2] eq "Topic") {
      $subname = "Subtopics";
    }

    (my $entriesInMod, my $moduleOverview) =
      createOnlineEntries("Perl_$$module[0]_", "", $subname, undef, 0, ${$module}[1]);
    my $underline = "*" x length($$module[2].": ".$$module[0]);

    my $intro = "";
    my $i = "";
    if($$module[3] ne "") {
      $i .= "$$module[3]";
    }
    if($$module[2] ne "Topic") {
      $intro .= "for help on a specific function type\n#help \\\"$$module[0]::function\\\"\n\n${i}";
    } else {
      if($entriesInMod > 0) {
	$intro .= "for help on a specific subtopic type\n#help \\\"$$module[0]: subtopic\\\"\n\n${i}";
      } else {
	$intro .= "${i}";
      }
    }
    print <<EOS;
  } elsif(\$helpOn eq "$$module[0]") {
    statusWindow()->print("
$$module[2]: $$module[0]
$underline
$intro$moduleOverview");
EOS
    foreach my $function (@{${$module}[1]}) {

      my $fnString = "$$module[0]::$$function[0]( $$function[1] )";
      if($$module[2] eq "Topic") {
	$fnString = "$$module[0]: $$function[0]";
      }
      my $underline = "*" x length($fnString);
      my $introF = "";
      my $onWhat = "$$module[0]::$$function[0]";
      if($$module[2] ne "Topic") {
        $introF .= "Synopsis: $fnString\n\n";
      } else {
	$onWhat = "$$module[0]: $$function[0]";
      }
      my $body = subOnlineColors($$function[2]);
      print <<EOS;
  } elsif(\$helpOn eq "$onWhat") {
    statusWindow()->print("
$fnString
$underline

$body");
EOS
    }
  }

  foreach my $function (@topfunctions) {
    my $fnString = "$$function[0]( $$function[1] )";
    my $underline = "*" x length($fnString);
    my $body = subOnlineColors($$function[2]);
    print <<EOS;
  } elsif(\$helpOn eq "$$function[0]") {
    statusWindow()->print("
$fnString
$underline

$body");
EOS
  }
  print "  }\n";
  print "# ***    END OF SCRIPT GENERATED SECTION     ***\n";
  print "# **********************************************\n";
}


############################################################
############################################################
############################################################

sub createDocbookEntries {
  my @ret = ( 0, "" );
  my ($prefix, $type, $heading, $typeElement, $nameElement, $elements) = @_;

  my $numInLine = 0;
  $ret[1] = "<table><title>$heading</title><tgroup cols=\"3\"><tbody>\n<row>";
  foreach my $element (@$elements) {
    #print "  type: $$element[$typeElement]\n";
    if($type eq "" || $$element[$typeElement] eq $type) {
      if($numInLine > 2) {
	$numInLine = 0;
	$ret[1] .= "</row>\n<row><entry>";
      } else {
	$ret[1] .= "<entry>";
      }
      my $id = $$element[$nameElement];
      $id =~ s/ /_/g;
      $id =~ s/\#/SHARP/g;
      $id =~ s/%/PERCENT/g;
#      print "\"$prefix\" + \"$id\"\n\n";
      $ret[1] .= "<link linkend=\"$prefix$id\">$$element[$nameElement]</link></entry>";
      $numInLine++;
      $ret[0]++;
    }
  }
  $ret[1] .= "</row>\n</tbody></tgroup></table>\n";
  if($ret[0] == 0) {
    $ret[1] = "";
  }

  return @ret;
}

  

if($ARGV[0] eq "docbook") {
  print "<!-- **********************************************\n";
  print "     *** DONT EDIT, GENERATED BY DESC2ONLINE.PL ***\n-->\n";

  my %num;
  my %numString;
  for my $s (['Module', 'Modules'],
	     ['Class', 'Classes'],
	     ['Topic', 'Topics']) {
    my ($type, $heading) = @$s;
    ($num{$type}, $numString{$type}) =
      createDocbookEntries("Perl_", $type, $heading, 2, 0, \@modules);
  }

  (my $funInS, my $functionsS) =
    createDocbookEntries("Perl_", "", "Functions", undef, 0, \@topfunctions);
  print <<EOS;
<chapter id="perl-developers">
<title>Perl Developers Guide to Xpertmud</title>
<para>
$functionsS
$numString{'Class'}
$numString{'Module'}
$numString{'Topic'}
</para>
EOS

  foreach my $module (@modules) {
    my $id = "Perl_" . $$module[0];
    $id =~ s/ /_/g;

    my $subname = "Functions";
    if($$module[2] eq "Topic") {
      $subname = "Subtopics";
    }

    my $pref = "Perl_$$module[0]_";
    $pref =~ s/ /_/g;
    (my $entriesInMod, my $moduleOverview) =
      createDocbookEntries($pref, "", $subname, undef, 0, ${$module}[1]);
    my $intro = "";
    if($$module[4] ne "") {
      $intro .= "<para><programlisting>$$module[4]</programlisting></para>\n";
    }
    print <<EOS;
<sect1 id="$id">
<title>$$module[2]: $$module[0]</title>
$intro
<para>
$moduleOverview
</para>
EOS
    foreach my $function (@{${$module}[1]}) {

      my $fnString = "$$module[0]::$$function[0]( $$function[1] )";
      my $introF = "";
      if($$module[2] ne "Topic") {
        $introF .= "Synopsis: $fnString\n\n";
      }
      my $id = "Perl_$$module[0]_$$function[0]";
      $id =~ s/ /_/g;
      print <<EOS;
<sect2 id="$id">
<title>$$function[0]</title>
<para>
<programlisting>
$introF$$function[3]
</programlisting>
</para>
</sect2>
EOS
    }
    print <<EOS;
</sect1>
EOS
  }

  foreach my $function (@topfunctions) {
    my $idTail = $$function[0];
    $idTail =~ s/\#/SHARP/g;
    $idTail =~ s/%/PERCENT/g;
    my $fnString = "$$function[0]( $$function[1] )";
    print <<EOS;
<sect1 id="Perl_$idTail">
<title>$$function[0]</title>
<para>
<programlisting>
Synopsis: $fnString

$$function[3]
</programlisting>
</para>
</sect1>
EOS
  }

  print "</chapter>\n";
  print "<!-- ***    END OF SCRIPT GENERATED SECTION     ***\n";
  print "     ********************************************** -->\n";
}


