from __future__ import nested_scopes

import XM
import sys
import types
import re
import traceback
import weakref

from xmlparser import * #FIXED

isEcho = 1
localEcho = 1

triggers = []
aliases = []
keyBindings = []
timers = []
delayed = []

mtime = 0

BLACK = 0
RED = 1
GREEN = 2
YELLOW = 3
BLUE = 4
MAGENTA = 5
CYAN = 6
WHITE = 7

def setLocalEcho(state):
    global localEcho
    localEcho = state

def showKeyNames():
    addKeyBinding('#SHOWKEYS', '^(.*)$',
                  lambda m: ((statusWindow().setFGColor(2) or 1) and
                             (statusWindow().write(m.group(1)+"\n") or 1) and
                             (statusWindow().resetAttributes() or 1) and
                             ((re.match('........ X', m.group(1)) and
                               delKeyBinding('^#SHOWKEYS$')) or 1) and
                             0)
                  or 1, 1)
    statusWindow().write("Key name mode active, press \"x\" to\n get back to normal mode\n")

def addIncludeDir(dir):
    sys.path.append(dir)

def prs(text):
    statusWindow().write(text)

psw = prs
                  
def statusWindow():
    global __statusWindow
    if(globals().has_key('__statusWindow')):
        return __statusWindow
    else:
        __statusWindow = XMTextBufferWindow(0)
        return __statusWindow

############################################################
# Trigger functions
############################################################
def addTrigger(name, regExp, toExp, flag=0, active=1):
    global triggers

    desc = "%s: " % name
    if callable(toExp):
        desc += "/%s/ -> function(): ..." % regExp
    else:
        desc += "s/%s/%s/g" % (regExp, str(toExp))

    def subRef(text): 
        (text, repCount) = re.compile(regExp).subn(toExp, text)
        if(repCount > 0):
#            statusWindow().write("\"%s\"\n" % text)
            if text != None and re.compile(r'^$').match(text):
                text = None
            return ( flag, text )
        else:
            return ( 0, text )
            
    triggers =  [[ name, desc, subRef, active ]] + triggers


def delTrigger(regExp):
    global triggers
    l = len(triggers)
    
    def triggerFilter(a):
        return ( re.compile(regExp).search(a[0]) == None )
    triggers = filter(triggerFilter, triggers)
    
    diff = l - len(triggers)
    return diff


def setTriggerState(regExp, active):
    count = 0
    def triggerMap(a): 
        if(re.compile(regExp).search(a[0]) != None):
            ++count
            a[3] = active
    
    map(triggerMap, triggers)
    return count


def listTriggers():
    statusWindow().setIntensive(1)
    statusWindow().write("Trigger list:\n")
    statusWindow().setIntensive(0)
    for trigger in triggers:
        [ name, desc, subRef, active ] = trigger
        
        statusWindow().write(desc)
        if(active):
            statusWindow().setFGColor(GREEN)
            statusWindow().write(" *ACTIVE*\n")
        else:
            statusWindow().setFGColor(RED)
            statusWindow().write(" *INACTIVE*\n")
        statusWindow().resetAttributes()

    statusWindow().write("\n")


def executeTriggers(text):
    for trigger in triggers:
        [ name, desc, subRef, active ] = trigger
        doBreak = 0
        if active and text != None:
            ( doBreak, text ) = subRef(text)
            
        if(doBreak):
            break
  
    return text

############################################################
# Alias functions
############################################################
def addAlias(name, regExp, toExp, flag=0, active=1):
    global aliases

    desc = "%s: " % name
    if callable(toExp):
        desc += "/%s/ -> function(): ..." % regExp
    else:
        desc += "s/%s/%s/g" % (regExp, str(toExp))

    def subRef(text): 
        (text, repCount) = re.compile(regExp).subn(toExp, text)
        if(repCount > 0):
            return ( flag, text )
        else:
            return ( 0, text )
            
    aliases =  [[ name, desc, subRef, active ]] + aliases


def delAlias(regExp):
    global aliases
    l = len(aliases)
    
    def aliasFilter(a):
        return ( re.compile(regExp).search(a[0]) == None )
    aliases = filter(aliasFilter, aliases)
    
    diff = l - len(aliases)
    return diff


def setAliasState(regExp, active):
    count = 0
    def aliasMap(a): 
        if(re.compile(regExp).search(a[0]) != None):
            ++count
            a[3] = active
    
    map(aliasMap, aliases)
    return count


def listAliases():
    statusWindow().setIntensive(1)
    statusWindow().write("Alias list:\n")
    statusWindow().setIntensive(0)
    for alias in aliases:
        [ name, desc, subRef, active ] = alias
        
        statusWindow().write(desc)
        if(active):
            statusWindow().setFGColor(GREEN)
            statusWindow().write(" *ACTIVE*\n")
        else:
            statusWindow().setFGColor(RED)
            statusWindow().write(" *INACTIVE*\n")
        statusWindow().resetAttributes()
    statusWindow().write("\n")


def executeAliases(text):
    for alias in aliases:
        [ name, desc, subRef, active ] = alias
        doBreak = 0
        if(active):
            ( doBreak, text ) = subRef(text)
            
        if(doBreak):
            break
  
    return text

############################################################
# KeyBinding functions
############################################################
def addKeyBinding(name, onKey, action, active=1):
    global keyBindings

    desc = "%s: " % name
    if callable(action):
        desc += "/%s/ -> function(): ..." % onKey
    else:
        desc += "/%s/ -> \"%s\"" % (onKey, action)

    if(not callable(action)):
        def subRef(match):
            wrap_onTextEntered(action)
    else:
        subRef = action
            
    keyBindings =  [[ name, desc, onKey, subRef, active ]] + keyBindings


def delKeyBinding(regExp):
    global keyBindings
    l = len(keyBindings)
    
    def keyBindingFilter(a):
        return ( re.compile(regExp).search(a[0]) == None )
    keyBindings = filter(keyBindingFilter, keyBindings)
    
    diff = l - len(keyBindings)
    return diff


def setKeyBindingState(regExp, active):
    count = 0
    def keyBindingMap(a): 
        if(re.compile(regExp).search(a[0]) != None):
            ++count
            a[4] = active
    
    map(keyBindingMap, keyBindings)
    return count


def listKeyBindings():
    statusWindow().setIntensive(1)
    statusWindow().write("Key binding list:\n")
    statusWindow().setIntensive(0)
    for keyBinding in keyBindings:
        [ name, desc, onKey, subRef, active ] = keyBinding
        
        statusWindow().write(desc)
        if(active):
            statusWindow().setFGColor(GREEN)
            statusWindow().write(" *ACTIVE*\n")
        else:
            statusWindow().setFGColor(RED)
            statusWindow().write(" *INACTIVE*\n")
        statusWindow().resetAttributes()
    statusWindow().write("\n")


def executeKeyBindings(key):
    for keyBinding in keyBindings:
        [ name, desc, onKey, subRef, active ] = keyBinding
        if(active):
            match = re.compile(onKey).match(key)
            if(match):
                subRef(match)
                return 1
  
    return 0

############################################################
# Timer functions
############################################################
def addTimer(name, inc, action, active=1):
    global timers

    desc = ("%s: +" % (name)) + str(inc) + " -> "

    if(not callable(action)):
        desc += "\"" + action + "\""
        def subRef():
            wrap_onTextEntered(action)
    else:
        subRef = action
        desc += "function(): ..."
            
    timers =  [[ name, desc, inc, subRef, active, 0 ]] + timers


def delTimer(regExp):
    global timers
    l = len(timers)
    
    def timerFilter(a):
        return ( re.compile(regExp).search(a[0]) == None )
    timers = filter(timerFilter, timers)
    
    diff = l - len(timers)
    return diff


def setTimerState(regExp, active):
    count = 0
    def timerMap(a): 
        if(re.compile(regExp).search(a[0]) != None):
            ++count
            a[4] = active
    
    map(timerMap, timers)
    return count

def setTimerInc(regExp, newInc):
    count = 0
    def timerMap(a): 
        if(re.compile(regExp).search(a[0]) != None):
            ++count
            a[2] = newInc
            a[1] = ("%s: +" % (a[0])) + str(newInc) + " -> "
            a[1] += str(a[3])
            
    map(timerMap, timers)
    return count


def listTimers():
    statusWindow().setIntensive(1)
    statusWindow().write("Timer list:\n")
    statusWindow().setIntensive(0)
    for timer in timers:
        [ name, desc, inc, subRef, active, last ] = timer
        
        statusWindow().write(desc)
        if(active):
            statusWindow().setFGColor(GREEN)
            statusWindow().write(" *ACTIVE*\n")
        else:
            statusWindow().setFGColor(RED)
            statusWindow().write(" *INACTIVE*\n")
        statusWindow().resetAttributes()
        
    statusWindow().write("\n")


# ?????? mtime correct ? time was never defined and will never be defined
def getTime():
    return mtime

def parse(file):
    found = 0
    for dir in [ '.' ] + sys.path:
        try:
            execfile(dir + "/" + file, globals(), globals())
            found = 1
            break
        except IOError:
            pass
        
    if(not found):
        XM.showError('File not found: "%s"\n' % file)


############################################################
# Delayed functions
############################################################
def addDelayed(name, delay, action):
    global delayed

    desc = ("%s: +" % (name)) + str(delay) + " -> "

    if(not callable(action)):
        desc += "\"" + action + "\""
        def subRef():
            wrap_onTextEntered(action)
    else:
        subRef = action
        desc += "function(): ..."
            
    delayed =  [[ name, desc, delay+getTime(), subRef ]] + delayed


def delDelayed(regExp):
    global delayed
    l = len(delayed)
    
    def delayedFilter(a):
        return ( re.compile(regExp).search(a[0]) == None )
    delayed= filter(delayedFilter, delayed)
    
    diff = l - len(delayed)
    return diff

def setDelayed(regExp,delay):
    global delayed
    count = 0
    delay
    def delayMap(a): 
        if(re.compile(regExp).search(a[0]) != None):
            ++count
            a[2] = delay+getTime()
            a[1] = ("%s: +" % (a[0])) + str(delay) + " -> "
            a[1] += "\"" + str(a[3]) + "\""
    
    map(delayMap, delayed)
    return count
    

def listDelayed():
    global delayed
    statusWindow().setIntensive(1)
    statusWindow().write("Delayed action list:\n")
    statusWindow().setIntensive(0)
    for dela in delayed:
        a = dela
        
        statusWindow().write( ("%s: +" % (a[0])) + str(a[2]-getTime()) + " -> "
                              "\"" + str(a[3]) + "\"" + "\n")
    statusWindow().resetAttributes()
    statusWindow().write("\n")


############################################################
# Help
############################################################
helps = {}

def help(helpOn = "", base = helps, preDesc = ""):
    if re.match(base['name'], helpOn) != None:
        if base['name'] == helpOn:
            prs("""
%s
%s

%s
""" % (preDesc+base['heading'],
       '*' * len(preDesc+base['heading']), base['description']))
            if base['subs'] != None:
                for key in base['subs']:
                    prs(key + "\n")
        else:
            helpOn = re.sub('[ \.]', '', re.sub(base['name'], '', helpOn))
            prs(helpOn + "\n")
            if base['subs'] != None:
                for key in base['subs'].keys():
                    for sub in base['subs'][key]:
                        if base['type'] == 'class' or base['type'] == 'module':
                            help(helpOn, sub,
                                 preDesc + base['name'] + ".")
                        elif base['type'] == 'collection':
                            help(helpOn, sub, "")
                        else:
                            # TODO
                            help(helpOn, sub, "")
                
                   


############################################################
# Callbacks
############################################################

def sysinitInitialize():
    addAlias("Eval", r'(?s)^.*$', __scripting, 1)
    addAlias("Compose", r'^.*$', __compose, 0)
    addAlias('Easy-Input', r'^%.*$', __onEasyInput, 1)
    
    addKeyBinding('Scroll-Up', '........ PgUp',
                  lambda m: ((statusWindow().scrollLines(
                               -statusWindow().getLines()+2) and 0) or 1))
    addKeyBinding('Scroll-Down', '........ PgDown',
                  lambda m: ((statusWindow().scrollLines(
                               statusWindow().getLines()-2) and 0) or 1))

    statusWindow().write("""
    
Welcome to xpertmud
*******************

type \"#help()\" or \"%help\"
for further information.
The # operator: \"#help(\"#\")\", or \"%help #\"
The % operator: \"#help(\"%\")\", or \"%help %\"

""")

def onTimer():
    global mtime
    global delayed
    if(mtime == 0.0):
        sysinitInitialize()
        
    mtime += 0.25

    for timer in timers:
        [ name, desc, inc, action, active, lastCall ] = timer
        if(active and (lastCall + inc < mtime)):
            timer[5] = mtime
            action()

    for dela in delayed:
        [name, desc, delay, subRef ] = dela
        if(delay <= mtime):
            subRef()
    ndela = []
    for dela in delayed:
        if(dela[2] > mtime):
            ndela = ndela + [dela]
    delayed = ndela
            

aliasPreprocessing = 0
def onTextEntered(text):
    if(callable(aliasPreprocessing)):
        text = aliasPreprocessing(text)
    
    text = executeAliases(text)
    
    if(text != ""):
        XM.send(text + "\r\n")
        if isEcho and localEcho:
            statusWindow().setFGColor(CYAN)
            statusWindow().write(text + "\n")
            statusWindow().resetAttributes()

keyPreprocessing = 0
def onKeyPressed(key, ascii):
    if(callable(keyPreprocessing)):
        ( key, ascii ) = keyPreprocessing(key, ascii)
    
    if(key != ""):
        if(executeKeyBindings(key)):
            return 1
        
        if(isEcho):
            return 0
        else:
            sendKey(key, ascii)
            return 1
    return 0

def onConnectionEstablished(id=0):
    statusWindow().write("Connection %d established.\n" % id)

def onConnectionDropped(id=0):
    statusWindow().write("Connection %d dropped.\n" % id)

writeAtOnceRegExp = r'.?'
dontWriteAtOnceRegExp = r'^$'
def setWriteRegExps(waoRegExp, dwaoRegExp):
    global writeAtOnceRegExp
    global dontWriteAtOnceRegExp
    writeAtOnceRegExp = waoRegExp or writeAtOnceRegExp
    dontWriteAtOnceRegExp = dwaoRegExp or dontWriteAtOnceRegExp

def resetWriteRegExps():
    global writeAtOnceRegExp
    global dontWriteAtOnceRegExp
    writeAtOnceRegExp = r'.?'
    dontWriteAtOnceRegExp = r'^$'
    
triggerPreprocessing = 0
onLineReceived = 0
linePart = ""
resetWriteRegExps()
def onTextReceived(text, id=0):
    global linePart

    if(callable(triggerPreprocessing)):
        text = triggerPreprocessing(text)

    toProcess = linePart + text
    if(toProcess != None and toProcess != ""):
        for line in toProcess.splitlines(1):
            linePart = ""
            (line, count) = re.compile(r'\r?\n$').subn(r'\n', line)
            if(count > 0):
                if(callable(onLineReceived)):
                    line = onLineReceived(line)

                line = executeTriggers(line)

                if(line != None and line != ""):
                    statusWindow().write(line)
            else:
                if((re.compile(writeAtOnceRegExp).search(line) != None) and
                   (re.compile(dontWriteAtOnceRegExp).search(line) == None)):
                    line = executeTriggers(line)
                    if(line != None and line != ""):
                        statusWindow().write(line)
                else:
                    linePart = line

def onEcho(state, id=0):
    global isEcho
    isEcho = state
    XM.echoMode(state)

def wrap_onEcho(state, id=0):
    try:
        if(callable(onEcho)):
            onEcho(state, id)
    except:
        dumpError(sys.exc_info())
    
def wrap_onTextEntered(text):
    try:
        if(callable(onTextEntered)):
            onTextEntered(text)
    except:
        dumpError(sys.exc_info())

def wrap_onTimer():
    try:
        if(callable(onTimer)):
            onTimer()
    except:
        dumpError(sys.exc_info())

def wrap_onKeyPressed(key, ascii):
    try:
        if(callable(onKeyPressed)):
            return onKeyPressed(key, ascii)
    except:
        dumpError(sys.exc_info())
    return 0

def wrap_onTextReceived(line, id=0):
    try:
        if(callable(onTextReceived)):
            onTextReceived(line,id)
    except:
        dumpError(sys.exc_info())

def wrap_onConnectionEstablished(id):
    try:
        if(callable(onConnectionEstablished)):
            onConnectionEstablished(id)
    except:
        dumpError(sys.exc_info())

def wrap_onConnectionDropped(id):
    try:
        if(callable(onConnectionDropped)):
            onConnectionDropped(id)
    except:
        dumpError(sys.exc_info())

def wrap_mouseDown(id, x, y):
    win = getWindow(id)
    if win != None:
        win.onMouseDown(x, y)

def wrap_mouseDrag(id, x, y):
    win = getWindow(id)
    if win != None:
        win.onMouseDrag(x, y)

def wrap_mouseUp(id, x, y):
    win = getWindow(id)
    if win != None:
        win.onMouseUp(x, y)

def wrap_pluginCall(id, function, args):
    win = getWindow(id)
    if win != None:
        return win.pluginCall(function, args)

def sendKey(key, ascii):
    if(ascii != None and re.compile(r'^.$').match(ascii) != None):
        XM.send(ascii)

allWindows = {}
def getWindow(id):
    if allWindows.has_key(id):
        return allWindows[id]
    else:
        return None

class XMWindow:
    def __del__(self):
        self.close()

    def register(self):
        allWindows[self.__id] = weakref.proxy(self)
    
    def setID(self, id):
        self.__id = id

    def getID(self):
        return self.__id

    def setTitle(self, text):
        XM.Window_setTitle(self.getID(), text)

    def show(self):
        XM.Window_show(self.getID())

    def hide(self):
        XM.Window_hide(self.getID())

    def close(self):
        import XM
        XM.Window_close(self.getID())

    def minimize(self):
        XM.Window_minimize(self.getID())

    def maximize(self):
        XM.Window_maximize(self.getID())

    def move(self, x, y):
        XM.Window_move(self.getID(), x, y)

    def resize(self, x, y):
        XM.Window_resize(self.getID(), x, y)

    def isValid(self):
        return XM.Window_isValid(self.getID())

    def raiseWin(self):
        XM.Window_raise(self.getID())

    def lowerWin(self):
        XM.Window_lower(self.getID())

    def getPositionX(self):
        return XM.Window_getPositionX(self.getID())

    def getPositionY(self):
        return XM.Window_getPositionY(self.getID())

    def getSizeX(self):
        return XM.Window_getSizeX(self.getID())

    def getSizeY(self):
        return XM.Window_getSizeY(self.getID())

    def showCaption(self):
        XM.Window_showCaption(self.getID())

    def hideCaption(self):
        XM.Window_hideCaption(self.getID())

class XMTextWindow (XMWindow):
    def __init__(self, id = None):
        if(id != None and type(id) != str):
            self.setID(id)
        else:
            self.setID(XM.TextWindow_initialize())
            if(id != None):
                self.setTitle(id)
        self.initTextWindow()

    def initTextWindow(self):
        self.onMouseDownFuncs = []
        self.onMouseDragFuncs = []
        self.onMouseUpFuncs = []
        self.register()

    def getCursorPos(self):
        return (self.getCursorX(), self.getCursorY())

    def getSizeChars(self):
        return (self.getColumns(), self.getLines())

    def registerMouseDownEvent(self, func):
        self.onMouseDownFuncs += [ func ]

    def registerMouseDragEvent(self, func):
        self.onMouseDragFuncs += [ func ]

    def registerMouseUpEvent(self, func):
        self.onMouseUpFuncs += [ func ]

    def onMouseDown(self, x, y):
        for func in self.onMouseDownFuncs:
            func(x, y)
            
    def onMouseDrag(self, x, y):
        for func in self.onMouseDragFuncs:
            func(x, y)

    def onMouseUp(self, x, y):
        for func in self.onMouseUpFuncs:
            func(x, y)

    def newline(self):
        XM.TextWindow_newline(self.getID())

    def clear(self):
        XM.TextWindow_clear(self.getID())

    def clearLine(self):
        XM.TextWindow_clearLine(self.getID())

    def clearEOL(self):
        XM.TextWindow_clearEOL(self.getID())

    def clearBOL(self):
        XM.TextWindow_clearBOL(self.getID())

    def clearEOS(self):
        XM.TextWindow_clearEOS(self.getID())

    def clearBOS(self):
        XM.TextWindow_clearBOS(self.getID())

    def resetAttributes(self):
        XM.TextWindow_resetAttributes(self.getID())

    def writeRaw(self, text):
        XM.TextWindow_printRaw(self.getID(), text)

    def write(self, text):
        XM.TextWindow_print(self.getID(), text)

    def setBGColor(self, color):
        XM.TextWindow_setBGColor(self.getID(), color)

    def setFGColor(self, color):
        XM.TextWindow_setFGColor(self.getID(), color)

    def setDefaultBGColor(self, color):
        XM.TextWindow_setDefaultBGColor(self.getID(), color)

    def setDefaultFGColor(self, color):
        XM.TextWindow_setDefaultFGColor(self.getID(), color)

    def scrollLines(self, num):
        XM.TextWindow_scrollLines(self.getID(), num)

    def scrollColumns(self, num):
        XM.TextWindow_scrollColumns(self.getID(), num)

    def setBold(self, state):
        XM.TextWindow_setBold(self.getID(), state)

    def setBlinking(self, state):
        XM.TextWindow_setBlinking(self.getID(), state)

    def setUnderline(self, state):
        XM.TextWindow_setUnderline(self.getID(), state)

    def setReverse(self, state):
        XM.TextWindow_setReverse(self.getID(), state)

    def setIntensive(self, state):
        XM.TextWindow_setIntensive(self.getID(), state)

    def resizeChars(self, x, y):
        XM.TextWindow_resizeChars(self.getID(), x, y)

    def setCursor(self, x, y):
        XM.TextWindow_setCursor(self.getID(), x, y)

    def getCursorX(self):
        return XM.TextWindow_getCursorX(self.getID())

    def getCursorY(self):
        return XM.TextWindow_getCursorY(self.getID())

    def getColumns(self):
        return XM.TextWindow_getColumns(self.getID())

    def getLines(self):
        return XM.TextWindow_getLines(self.getID())

    def getCharAt(self, x, y):
        return XM.TextWindow_getCharAt(self.getID(), x, y)

    def setFont(self, name, size):
        return XM.TextWindow_setFont(self.getID(), name, size)

    def getFGColorAt(self, x, y):
        return XM.TextWindow_getFGColorAt(self.getID(), x, y)

    def getBGColorAt(self, x, y):
        return XM.TextWindow_getBGColorAt(self.getID(), x, y)

    def enableMouseEvents(self):
        XM.TextWindow_enableMouseEvents(self.getID(), )

    def disableMouseEvents(self):
        XM.TextWindow_disableMouseEvents(self.getID(), )

    def getWordWrapColumn(self):
        XM.TextBufferWindow_getWordWrapColumn(self.getID())

    def setWordWrapColumn(self, c):
        XM.TextBufferWindow_setWordWrapColumn(self.getID(), c)

class XMTextBufferWindow (XMTextWindow):
    def __init__(self, id = None):
        if(id != None and type(id) != str):
            self.setID(id)
        else:
            self.setID(XM.TextBufferWindow_initialize())
            if(id != None):
                self.setTitle(id)
        self.initTextWindow()

class XMPlugin (XMWindow):
    def __init__(self, lib, cls, title):
        id = XM.Plugin_initialize(lib, cls)
        if id < 0:
            return None
        print id
        self.setID(id)
        print self.getID()
        self.setTitle(title)
        self.register()

    def pluginCall(self, functions, args):
        return None
        

def dumpError(excInfo):
    """ Write a nice error
    message; TODO: Nice output!
    """
    errStr = "Traceback (most recent call last):\n"
    for line in traceback.format_tb(excInfo[2]):
            errStr += line
            
    excName = re.sub(r'exceptions\.', r'', str(excInfo[0]))
    errStr += excName + ": " + str(excInfo[1]) + "\n"
    
    print errStr
    XM.showError(errStr)

class RegState:
    def perlSub(self, regExp, toExp, string):
        self.match = re.search(regExp, string)
        (self.string, self.count) = re.subn(regExp, toExp, string)
        return (self.count > 0)

reS = RegState()

def __scripting(match):
    if(re.compile(r'^#[^#]').match(match.string)):
        exec match.string[1:] in globals()
        return ""
    else:
        return match.string

composeBuffer = ""
def __compose(match):
    global composeBuffer
    if(re.compile(r'^##$').match(match.string)):
        composeBuffer += "\n"
        print "execing %s" % composeBuffer
        exec composeBuffer in globals()
        composeBuffer = ""
        return ""
    elif(re.compile(r'^##').match(match.string)):
        composeBuffer += match.string[2:] + "\n"
        statusWindow().setFGColor(BLUE)
        statusWindow().write("> " + match.string[2:] + "\n")
        statusWindow().resetAttributes()
        return ""
    else:
        return match.string

def __onEasyInput(match):
    command = ""
    str = match.string[1:]
    r = re.compile(r'^\s*([^\s]+)\s*')
    m = r.match(str)
    if m != None:
        command += m.group(1) + "("
        str = r.sub('', str)

    r = re.compile(r'(?ix)^'
                   r'(("([^"\\]|(\\")|(\\[^"]))*") |' # " ... " pairs
                   r'(r?\'([^\'\\]|(\\\')|(\\[^\']))*\') |' # r?' ... ' pairs
                   r'([^"][^ ]*))\s*') # wordswithoutspaces
    m = r.match(str)
    if m != None:
        print m.groups()
        print m.group(1)
        print m.group(2)
        if (m.group(2) != None) or (m.group(6) != None):
            command += m.group(1)
        else:
            command += '"' + m.group(1) + '"'
        str = r.sub('', str)
        m = r.match(str)
    
    while m != None:
        print m.groups()
        command += ', '
        if (m.group(2) != None) or (m.group(6) != None):
            command += m.group(1)
        else:
            command += '"' + m.group(1) + '"'
        str = r.sub('', str)
        m = r.match(str)

    command += ")\n"
    print "execing %s" % command
    exec command in globals()
    
    
